//Copyright 2018 rat de combat sur forum.hardware.fr

/*
This file is part of the firmware for the AD9833-poor men's signal generator.

The firmware is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The firmware is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the firmware.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>

#include "ad9833.h"
#include "parser.h"

#define BUFFER_FORMAT_FREQ (12+1)

#define SYMBOL_DEGREE '\xb0'

static waveform_t waveform=SINE;
static uint32_t frequency=FREQUENCY_DEFAULT;
static uint16_t phase=PHASE_DEFAULT;

uint8_t parse_frequency(char * const buf, uint32_t * const f);
uint8_t parse_phase(char * const buf, uint16_t * const p); 
void format_freq(const uint32_t f, char * const buf);

void reset_all(void)
{
	reset_ad9833();
	write_waveform(SINE);
	write_freq(frequency, REG0);
	//write_freq(frequency2, REG1);
	write_phase(phase, REG0);
	//write_phase(phase2, REG1);
	select_register(FREQ, REG0);
	select_register(PHASE, REG0);
}

void parse_command(char * const str)
{
	const char * str_waveforms[3]={"sine", "triangle", "square"};
	
	char str_freq[BUFFER_FORMAT_FREQ];
	
	format_freq(frequency, str_freq); //convert %f to %s
	
	if(strlen(str)==0)
	{
		return;
	}

	if(strncmp(str, "reset", strlen("reset"))==0)
	{
		reset_all();
		waveform=SINE;
		frequency=FREQUENCY_DEFAULT;
		phase=PHASE_DEFAULT;
		printf("reset ok\r\n");
	}
	else if(strncmp(str, "?", strlen("?"))==0 || strncmp(str, "help", strlen("help"))==0)
	{
		printf("valid commands:\r\nreset\r\nhelp or ?\r\nsine\r\ntriangle\r\nsquare\r\n<frequency with suffix none or k or M>\r\n<phase with suffix %c>\r\n", SYMBOL_DEGREE);
	}
	else if(strncmp(str, "sine", strlen("sine"))==0)
	{
		write_waveform(SINE);
		waveform=SINE;
		printf("sine selected with %s %d°, use output 2\r\n", str_freq, phase);
	}
	else if(strncmp(str, "square", strlen("square"))==0)
	{
		write_waveform(SQUARE);
		waveform=SQUARE;
		printf("square selected with %s %d°, use output 1\r\n", str_freq, phase);
	}
	else if(strncmp(str, "triangle", strlen("triangle"))==0)
	{
		write_waveform(TRIANGLE);
		waveform=TRIANGLE;
		printf("triangle selected with %s %d°, use output 2\r\n", str_freq, phase);
		if(frequency>1000000)
		{
			frequency=1000000;
			write_freq(frequency, REG0);
			printf("frequency clipped to 1MHz\r\n");
		}
	}			
	else if(strncmp(str, "status", strlen("status"))==0)
	{
		printf("waveform: %s  frequency: %s  phase: %d°\r\n", str_waveforms[waveform], str_freq, phase);
	}
	else if(parse_frequency(str, &frequency)) //may modify buffer!
	{
		if(((waveform==SINE || waveform==SQUARE) && frequency<=10000000) || (waveform==TRIANGLE && frequency<=1000000))
		{
			write_freq(frequency, REG0);
			format_freq(frequency, str_freq);
			printf("frequency switched to %s\r\n", str_freq);
		}
		else
			printf("error invalid frequency\r\n");
	}
	else if(parse_phase(str, &phase)) //may modify buffer!
	{
		if(phase<=359)
		{
			write_phase(phase, REG0);
			printf("phase switched to %d°\r\n", phase);
		}
		else
			printf("error invalid phase\r\n");
	}
	else
	{
		printf("error: unknown command %s\r\n", str);
	}
}

uint8_t parse_frequency(char * const buf, uint32_t * const f) //reinventing the wheel
{
	uint32_t multiplier=1;
	uint32_t freq=0;
	
	if(buf[strlen(buf)-1]==SYMBOL_DEGREE) //phase
	{
		return 0;
	}
	else if(buf[strlen(buf)-1]=='k')
	{
		
		multiplier=1000;
		buf[strlen(buf)-1]='\0';
	}
	else if(buf[strlen(buf)-1]=='M')
	{
		multiplier=1000000;
		buf[strlen(buf)-1]='\0';
	}
	
	uint8_t i;
	for(i=0; i<strlen(buf); i++)
	{
		if(isdigit(buf[i])==0 && buf[i]!='.')
			return 0;
	}
	
	uint8_t dot=0;
	for(i=0; i<strlen(buf); i++)
	{
		if(buf[i]!='.')
		{
			if(!dot)
			{
				freq*=10;
				freq+=buf[i]-'0';
			}
			else
			{
				multiplier/=10;
				if(multiplier)
				{
					freq*=10;
					freq+=buf[i]-'0';
				}
			}
		}
		else
		{
			dot=1;
		}
	}
	
	if(multiplier==0)
	{
		multiplier=1;
		printf("warning: decimals ignored\r\n");
	}
	
	*f=freq*multiplier;
	
	return 1;
}

uint8_t parse_phase(char * const buf, uint16_t * const p) 
{
	if(buf[strlen(buf)-1]!=SYMBOL_DEGREE) //phase
	{
		return 0;
	}
	buf[strlen(buf)-1]='\0';
	
	*p=atoi(buf);
	
	return 1;
}
void format_freq(const uint32_t f, char * const buf)
{
	memset(buf, 1, 13);
	if(f>=1000000)
	{
		sprintf(buf, "%lu.%06luMHz", f/1000000, f%1000000);
	}
	else if(f>=1000)
	{
		sprintf(buf, "%lu.%03lukHz", f/1000, f%1000);
	}
	else
	{
		sprintf(buf, "%luHz", f);
	}
}
